## Praat script for Plotnik "button" to automatically substitute Praat formant measurements
## for all given/visible tokens more than 1 standard deviation from the F1/F2 mean
## written by Ingrid Rosenfelder
## last updated April 26, 2010

## input:
## name/path of sound file:  path_of_sound_file$
## time of measurement: time_of_measurement
## formant settings:  numF, maxF
form Get input file names and time of measurement
	sentence Path_of_sound_file Howard:Users:ingrid:Documents:FAAV:analysis_procedures:vowel_analysis:PlotnikButtons:testbutton.wav
	real Time_of_measurement 1234.088
	integer numF 5
	integer maxF	 5500
endform

############################ PROCEDURE DEFINITIONS###############################################

procedure convert_path_to_Unix_format .macpath$ .unixpath$
	## converts Mac path names (with ":" as path separator) to Unix conventions (with "/" as path separator)
	## and replaces the name of the hard disk in an absolute Mac path (e.g. "Max:") with a Unix root slash ("/")
	## NOTE:  this only works for absolute paths; otherwise the first directory in the path will be chopped off!
	.index_first_colon = index(.macpath$, ":")
	'.unixpath$' = "/" + right$(replace$(.macpath$, ":", "/", 0), length(.macpath$) - .index_first_colon)
endproc

procedure name_in_objects_list .path$ .name$ .type$
	## converts absolute path name of file to be opened into name of that file in the objects list
	## 1) chop off all path prefixes
	last_slash = rindex(.path$, "/")
	.name1$ = right$(.path$, length(.path$) - last_slash)
	## 2) convert special characters in filename
	## \W = not a word
	## (everything that is not [a-zA-Z0-9_] or "-" (single hyphen/minus) is replaced by underscores)
	if '.type$' = "LongSound" or '.type$' = "Sound"
		'.name$' = replace_regex$(.name1$ - ".wav", "\W-", "_", 0)
	elsif '.type$' = "TextGrid"
		'.name$' = replace_regex$(.name1$ - ".TextGrid", "\W-", "_", 0)
	else
		pause Unknown file type!  ('.type')
	endif
	## (NOTE:  this could apparently also be achieved with name$ = extractLine$(fullName$, " "))
endproc

procedure in_objects_list .name$ .isopen$ .type$
	## checks whether a given file of a certain type is already in the objects list
	## (type can be "LongSound", "Sound", "TextGrid" etc.)
	select all
	for n from 1 to numberOfSelected('.type$')
		if extractWord$(selected$('.type$', n), "") = .name$
			'.isopen$' = "1"
		else
			'.isopen$' = "0"
		endif
	endfor
endproc

############################ MAIN PROGRAM STARTS HERE...##########################################

## convert Mac paths to Unix format
	## -> unix_path_of_sound_file$
call convert_path_to_Unix_format 'path_of_sound_file$' unix_path_of_sound_file$

## get file names in objects list
	## -> name_of_sound_file_in_objects_list$
call name_in_objects_list 'unix_path_of_sound_file$' name_of_sound_file_in_objects_list$ "LongSound"

## check whether sound file and TextGrid are already in the object list
	## -> isopen_sound
isopen_sound$ = "0"
call in_objects_list 'name_of_sound_file_in_objects_list$' isopen_sound$ "LongSound"

## open sound file and corresponding TextGrid, if not in objects list already
if isopen_sound$ = "0"
	Open long sound file... 'unix_path_of_sound_file$'
endif

## cannot do any measurements on long sound file, so extract relevant part around time of measurement
## "yes" -> keep original times
extract_start = time_of_measurement - 0.1
extract_end = time_of_measurement + 0.1
select LongSound 'name_of_sound_file_in_objects_list$'
Extract part... extract_start extract_end yes

## get formants
To Formant (burg)... 0 numF maxF 0.025 50
ff1 = Get value at time... 1 time_of_measurement Hertz Linear
ff2 = Get value at time... 2 time_of_measurement Hertz Linear
f1 = round(ff1)
f2 = round(ff2)

## write output to file 
print 'f1','f2'

## remove objects in Praat objects list (would become cluttered fast!)
select Formant 'name_of_sound_file_in_objects_list$'
Remove
select Sound 'name_of_sound_file_in_objects_list$'
Remove

